#include <windows.h>
#include <windowsx.h>
#include <tchar.h>
#include "winutils.h"
#include "resource.h"

#ifndef IDANI_CAPTION
#define IDANI_CAPTION 3
#endif

#define MAXLEN 1000

static RECT  s_rcItem           = { 0 };
static TCHAR s_szItem[ MAXLEN ] = { 0 };

BOOL CALLBACK exitDlgProc(
    HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    if ( WM_INITDIALOG == msg ) {
        HWND hwndButton = (HWND) lParam;
        RECT rcButton = { 0 };
        RECT rcDialog = { 0 };
        GetWindowRect( hwnd      , &rcDialog );
        GetWindowRect( hwndButton, &rcButton );
        DrawAnimatedRects( hwnd, 
            IDANI_CAPTION, &rcButton, &rcDialog );
        return TRUE;
    }
    if ( WM_COMMAND == msg ) {
        EndDialog( hwnd, IDOK );
        return TRUE;
    }
    return FALSE;
}

BOOL CALLBACK detailDlgProc(
    HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    HWND hwndEdit = GetDlgItem( hwnd, IDC_EDIT );
    RECT rc = { 0 };

    if ( WM_INITDIALOG == msg ) {
        SetWindowText( hwndEdit, s_szItem );
        Edit_LimitText( hwndEdit, MAXLEN - 1 );
        SetWindowPos( hwnd, 0,
            s_rcItem.left, s_rcItem.bottom, 0, 0,
            SWP_NOSIZE | SWP_NOZORDER );
        adjustToWorkArea( hwnd );
        GetWindowRect( hwnd, &rc );
        DrawAnimatedRects( hwnd, IDANI_CAPTION, &s_rcItem, &rc );
        return TRUE;
    }
    if ( WM_COMMAND == msg ) {
        const int id = GET_WM_COMMAND_ID( wParam, lParam );
        switch ( id ) {
        case IDOK:
            GetWindowRect( hwnd, &rc );
            DrawAnimatedRects( hwnd, 
                IDANI_CAPTION, &rc, &s_rcItem );
            GetWindowText( hwndEdit, s_szItem, MAXLEN );
            //*** FALL THROUGH!

        case IDCANCEL:
            EndDialog( hwnd, id );
            return TRUE;
        }
    }
    return FALSE;
}

void doDetail( HWND hwnd ) {
    int nResult;
    HWND hwndList = GetDlgItem( hwnd, IDC_LIST );
    const int nSelIndex = ListBox_GetCurSel( hwndList );

    ListBox_GetText( hwndList, nSelIndex, s_szItem );
    ListBox_EnsureVisible( hwndList, nSelIndex );
    ListBox_GetItemRect( hwndList, nSelIndex, &s_rcItem );
    MapWindowPoints(
        hwndList, HWND_DESKTOP, (LPPOINT) &s_rcItem, 2 );
    nResult = DialogBox( GetWindowInstance( hwnd ),
        _T( "DetailDlg" ), hwnd, detailDlgProc );
    if ( IDOK == nResult ) {
        const int nTopIndex = ListBox_GetTopIndex( hwndList );
        SetWindowRedraw( hwndList, FALSE );
        ListBox_DeleteString( hwndList, nSelIndex );
        ListBox_InsertString( hwndList, nSelIndex, s_szItem );
        ListBox_SetCurSel   ( hwndList, nSelIndex );
        ListBox_SetTopIndex ( hwndList, nTopIndex );
        SetWindowRedraw( hwndList, TRUE );
    }
}

BOOL handleCommand(
    HWND hwnd, int id, HWND hwndCtl, UINT codeNotify )
{
    switch ( id ) {
    case IDC_LIST:
        if ( LBN_SELCHANGE == codeNotify ) {
            HWND hwndOK = GetDlgItem( hwnd, IDOK );
            EnableWindow( hwndOK, TRUE );
            Button_SetStyle( GetDlgItem( hwnd, IDOK ),
                BS_DEFPUSHBUTTON, TRUE );
        } else if ( LBN_DBLCLK == codeNotify ) {
            doDetail( hwnd );
        }
        break;
    case IDOK:
        doDetail( hwnd );
        return TRUE;
    case IDCANCEL:
        DialogBoxParam( GetWindowInstance( hwnd ),
            _T( "ExitDlg" ), hwnd, exitDlgProc,
            (LPARAM) GetDlgItem( hwnd, IDCANCEL ) );
        EndDialog( hwnd, IDCANCEL );
        return TRUE;
    }
    return FALSE;
}

BOOL CALLBACK mainDlgProc(
    HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    if ( WM_INITDIALOG == msg ) {
        HWND hwndList = GetDlgItem( hwnd, IDC_LIST );
        TCHAR sz[ 100 ] = { 0 };
        int iItem = 0;
        for ( iItem = 0; iItem < 20; ++iItem ) {
            wsprintf( sz, _T( "Item #%d" ), iItem );
            ListBox_AddString( hwndList, sz );
        }
        ListBox_SetCaretIndex( hwndList, 0 );
        Button_SetStyle( GetDlgItem( hwnd, IDOK ),
            BS_PUSHBUTTON, TRUE );
        return TRUE;
    }
    if ( WM_COMMAND == msg ) {
        return handleCommand( hwnd,
            GET_WM_COMMAND_ID  ( wParam, lParam ) ,
            GET_WM_COMMAND_HWND( wParam, lParam ) ,
            GET_WM_COMMAND_CMD ( wParam, lParam ) );
    }
    return FALSE;
}

int APIENTRY _tWinMain( HINSTANCE hinst,
    HINSTANCE hinstPrev, LPTSTR lpszCmdLine, int nCmdShow )
{
    return DialogBox( hinst, _T( "MainDlg" ), 0, mainDlgProc );
}

/* end of file */
