
// __________________________________________________________
//
//                         IsapiSim.c
//              ISAPI Extension Simulator V1.01
//                05-02-1999 Sven B. Schreiber
//                       sbs@orgon.com
// __________________________________________________________

#define  _ISAPISIM_DLL_
#include "IsapiSim.h"

// =================================================================
// DISCLAIMER
// =================================================================

/*

This software is provided "as is" and any express or implied
warranties, including, but not limited to, the implied warranties of
merchantibility and fitness for a particular purpose are disclaimed.
In no event shall the author Sven B. Schreiber be liable for any
direct, indirect, incidental, special, exemplary, or consequential
damages (including, but not limited to, procurement of substitute
goods or services; loss of use, data, or profits; or business
interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software,
even if advised of the possibility of such damage.

*/

// =================================================================
// REVISION HISTORY
// =================================================================

/*

03-17-1999 V1.00 Original version (SBS).

05-02-1999 V1.01 Update (SBS).

    Added support for ecb.lpszPathInfo and ecb.lpszPathTranslated.
    Added the command line parser IsapiTestQuery(). Before calling
    IsapiTestLoad(), the current directory is switched to the
    ISAPI DLL's program directory, and switched back after
    IsapiTestUnload(). Thus, the simulator can reside in an
    arbitrary directory without impairing the ISAPI DLLs ability to
    find all of its subordinate modules. Added HTTP header parsing
    to be able to map content types to output file extensions.
    Optimized IsapiTestWriteClient().

*/

// =================================================================
// GLOBAL VARIABLES
// =================================================================

HINSTANCE ghInstance = NULL;

// =================================================================
// GLOBAL STRINGS
// =================================================================

BYTE abName           [] = SA(MAIN_CAPTION);
BYTE abMethodGet      [] = "GET";
BYTE abMethodPost     [] = "POST";
BYTE abShellOpen      [] = "open";
BYTE abContentType    [] = "application/x-www-form-urlencoded";
BYTE abTagHttp        [] = "HTTP/";
BYTE abTagContentType [] = "Content-Type:";
BYTE abFileDefault    [] = ".html";
BYTE abNewLine        [] = "\r\n";
BYTE abNull           [] = "";

BYTE abHtmlStart      [] = SA(<HTML>\r\n<HEAD>)
                           SA(<TITLE>MAIN_CAPTION</TITLE>)
                           SA(</HEAD>\r\n<BODY><PRE>);

BYTE abHtmlStop       [] = SA(</PRE></BODY>\r\n</HTML>\r\n);


// =================================================================
// CONTENT TYPES
// =================================================================

ISAPI_VALUE aContentTypes [] =
    {
    "text/html",  ".html",
    "text/plain", ".txt",
    "image/jpeg", ".jpeg",
    "image/gif",  ".gif",
    "image/bmp",  ".bmp",
    NULL,         NULL,
    };

// =================================================================
// CONVERSION TABLES
// =================================================================

BYTE LCase [] =
    {
      0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
     16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31,
     32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47,
     48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63,
     64, 97, 98, 99,100,101,102,103,104,105,106,107,108,109,110,111,
    112,113,114,115,116,117,118,119,120,121,122, 91, 92, 93, 94, 95,
     96, 97, 98, 99,100,101,102,103,104,105,106,107,108,109,110,111,
    112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127,
    128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143,
    144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,255,
    160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175,
    176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191,
    224,225,226,227,228,229,230,231,232,233,234,235,236,237,238,239,
    240,241,242,243,244,245,246,215,248,249,250,251,252,253,222,223,
    224,225,226,227,228,229,230,231,232,233,234,235,236,237,238,239,
    240,241,242,243,244,245,246,247,248,249,250,251,252,253,222,255,
    };

// =================================================================
// HELPER ROUTINES
// =================================================================

INT WINAPI IsapiBox (HWND  hWnd,
                     UINT  uiType,
                     PBYTE pbCaption,
                     PBYTE pbFormat,
                     ...)
    {
    BYTE  abBuffer [1024];
    PVOID pArgs = &pbFormat + 1;

    wvsprintfA (abBuffer, pbFormat, pArgs);
    return MessageBoxA (hWnd, abBuffer, pbCaption, uiType);
    }

// -----------------------------------------------------------------

BOOL WINAPI IsapiCompare (BYTE bData,
                          BYTE bPattern,
                          BOOL fCase)
    {
    return (fCase ? (       bData  ==        bPattern )
                  : (LCase [bData] == LCase [bPattern]));
    }

// -----------------------------------------------------------------

BOOL WINAPI IsapiMatch (PBYTE pbData,
                        DWORD dData,
                        PBYTE pbPattern,
                        DWORD dPattern,
                        BOOL  fCase)
    {
    DWORD dData1, dPattern1, i;
    BOOL  fOk = FALSE;

    dData1    = (pbData == NULL
                 ? 0 : (dData != MAXDWORD
                        ? dData : lstrlenA (pbData)));

    dPattern1 = (pbPattern == NULL
                 ? 0 : (dPattern != MAXDWORD
                        ? dPattern : lstrlenA (pbPattern)));

    if (dData1 == dPattern1)
        {
        for (i = 0; i < dData1; i++)
            {
            if (!IsapiCompare (pbData [i], pbPattern [i], fCase))
                {
                break;
                }
            }
        fOk = (i == dData1);
        }
    return fOk;
    }

// -----------------------------------------------------------------

BOOL WINAPI IsapiCopy (PBYTE pbData,
                       DWORD dData,
                       PBYTE pbBuffer,
                       DWORD dBuffer)
    {
    DWORD dData1;
    DWORD n = 0;

    dData1 = (pbData == NULL
              ? 0 : (dData != MAXDWORD
                     ? dData : lstrlenA (pbData)));

    if ((pbBuffer != NULL) && dBuffer)
        {
        while (n < min (dData1, dBuffer-1))
            {
            pbBuffer [n] = pbData [n];
            n++;
            }
        pbBuffer [n++] = 0;
        }
    return n;
    }

// =================================================================
// MEMORY MANAGEMENT
// =================================================================

PVOID WINAPI IsapiMemoryCreate (DWORD dSize)
    {
    return LocalAlloc (LMEM_FIXED, max (dSize, 1));
    }

// -----------------------------------------------------------------

PVOID WINAPI IsapiMemoryResize (PVOID pData,
                                DWORD dSize)
    {
    DWORD dSize1, dSize2;
    PVOID pData1 = NULL;

    if (pData != NULL)
        {
        if (dSize1 = LocalSize (pData))
            {
            if ((dSize2 = max (dSize, 1)) != dSize1)
                {
                if ((pData1 = LocalReAlloc (pData, dSize2,
                                            LMEM_MOVEABLE))
                    == NULL)
                    {
                    LocalFree (pData);
                    }
                }
            else
                {
                pData1 = pData;
                }
            }
        else
            {
            LocalFree (pData);
            }
        }
    return pData1;
    }

// -----------------------------------------------------------------

PVOID WINAPI IsapiMemoryDestroy (PVOID pData)
    {
    if (pData != NULL) LocalFree (pData);
    return NULL;
    }

// -----------------------------------------------------------------

PVOID WINAPI IsapiMemoryCopy (PVOID pTarget,
                              PVOID pSource,
                              DWORD dBytes)
    {
    DWORD n = 0;

    if ((pTarget != NULL) && (pSource != NULL) && dBytes)
        {
        CopyMemory (pTarget, pSource, n = dBytes);
        }
    return (PBYTE) pTarget + n;
    }

// =================================================================
// FILE MANAGEMENT
// =================================================================

HANDLE WINAPI IsapiFileNew (PBYTE pbPath)
    {
    HANDLE hf = INVALID_HANDLE_VALUE;

    if (pbPath != NULL)
        {
        hf = CreateFileA (pbPath, GENERIC_READ | GENERIC_WRITE,
                          FILE_SHARE_READ, NULL, CREATE_ALWAYS,
                          FILE_FLAG_SEQUENTIAL_SCAN |
                          FILE_ATTRIBUTE_NORMAL,
                          NULL);
        }
    return hf;
    }

// -----------------------------------------------------------------

HANDLE WINAPI IsapiFileClose (HANDLE hf)
    {
    if (hf != INVALID_HANDLE_VALUE) CloseHandle (hf);
    return INVALID_HANDLE_VALUE;
    }

// -----------------------------------------------------------------

BOOL WINAPI IsapiFileSave (PBYTE pbPath,
                           PVOID pData,
                           DWORD dData)
    {
    HANDLE hf;
    DWORD  n;
    BOOL   fOk = FALSE;

    if ((hf = IsapiFileNew (pbPath)) != INVALID_HANDLE_VALUE)
        {
        n = 0;
        if ((pData != NULL) && dData)
            {
            if (!WriteFile (hf, pData, dData, &n, NULL)) n = 0;
            }
        fOk = (n == dData) && SetEndOfFile (hf);
        IsapiFileClose (hf);

        if (!fOk) DeleteFileA (pbPath);
        }
    return fOk;
    }

// -----------------------------------------------------------------

DWORD WINAPI IsapiFilePath (HINSTANCE hInstance,
                            PBYTE     pbFile,
                            PBYTE     pbBuffer,
                            DWORD     dBuffer)
    {
    DWORD i, n;
    DWORD dSize = 0;

    if ((pbBuffer != NULL) && dBuffer)
        {
        if ((n = GetModuleFileNameA ((hInstance != NULL
                                      ? hInstance
                                      : ghInstance),
                                     pbBuffer, dBuffer))
            &&
            (n < dBuffer))
            {
            if (pbFile != NULL)
                {
                for (i = n; i; i--)
                    {
                    if ((pbBuffer [i-1] == ':' ) ||
                        (pbBuffer [i-1] == '\\'))
                        {
                        break;
                        }
                    if ((pbFile   [ 0 ] == '.') &&
                        (pbBuffer [i-1] == '.'))
                        {
                        i--;
                        break;
                        }
                    }
                if ((n = i + lstrlenA (pbFile)) < dBuffer)
                    {
                    lstrcpyA (pbBuffer+i, pbFile);
                    dSize = n;
                    }
                }
            else
                {
                dSize = n;
                }
            }
        pbBuffer [dSize] = 0;
        }
    return dSize;
    }

// =================================================================
// HTTP HEADER MANAGEMENT
// =================================================================

BOOL WINAPI IsapiHeaderSpace (BYTE bData)
    {
    return (bData == ' ') || (bData == '\t');
    }

// -----------------------------------------------------------------

BOOL WINAPI IsapiHeaderEol (BYTE bData)
    {
    return (bData == '\r') || (bData == '\n');
    }

// -----------------------------------------------------------------

BOOL WINAPI IsapiHeaderStop (PBYTE pbData,
                             DWORD dData,
                             DWORD dOffset)
    {
    return (pbData == NULL) || (dOffset >= dData) ||
           IsapiHeaderEol (pbData [dOffset]);
    }

// -----------------------------------------------------------------

DWORD WINAPI IsapiHeaderToken (PBYTE pbData,
                               DWORD dData,
                               DWORD dOffset)
    {
    DWORD dOffset1 = dOffset;

    if (pbData != NULL)
        {
        while ((!IsapiHeaderStop (pbData, dData, dOffset1)) &&
               IsapiHeaderSpace (pbData [dOffset1]))
            {
            dOffset1++;
            }
        }
    return dOffset1;
    }

// -----------------------------------------------------------------

DWORD WINAPI IsapiHeaderNext (PBYTE pbData,
                              DWORD dData,
                              DWORD dOffset)
    {
    DWORD dOffset1 = dOffset;

    if (pbData != NULL)
        {
        while (!IsapiHeaderStop (pbData, dData, dOffset1))
            {
            dOffset1++;
            }
        if ((dOffset1 < dData) && (pbData [dOffset1] =='\r'))
            {
            dOffset1++;
            }
        if ((dOffset1 < dData) && (pbData [dOffset1] =='\n'))
            {
            dOffset1++;
            }
        }
    return dOffset1;
    }

// -----------------------------------------------------------------

DWORD WINAPI IsapiHeaderSkip (PBYTE pbData,
                              DWORD dData,
                              DWORD dOffset)
    {
    DWORD dOffset1 = dOffset;

    if (pbData != NULL)
        {
        while (!IsapiHeaderStop (pbData, dData, dOffset1))
            {
            dOffset1 = IsapiHeaderNext (pbData, dData, dOffset1);
            }
        dOffset1 = IsapiHeaderNext (pbData, dData, dOffset1);
        }
    return dOffset1;
    }

// -----------------------------------------------------------------

BOOL WINAPI IsapiHeaderTest (PBYTE  pbData,
                             DWORD  dData,
                             PDWORD pdOffset,
                             PBYTE  pbTag,
                             BOOL   fCase)
    {
    DWORD i;
    DWORD dOffset = (pdOffset != NULL ? *pdOffset : 0);
    DWORD dValue  = 0;

    if ((pbTag != NULL) && pbTag [0] &&
        (!IsapiHeaderStop (pbData, dData, dOffset)))
        {
        for (i = 0; pbTag [i]; i++)
            {
            if (IsapiHeaderStop (pbData, dData, dOffset)
                ||
                (!IsapiCompare (pbData [dOffset++], pbTag [i],
                                fCase)))
                {
                break;
                }
            }
        if (!pbTag [i]) dValue = dOffset;
        dOffset = IsapiHeaderNext (pbData, dData, dOffset);
        }
    if (pdOffset != NULL) *pdOffset = dOffset;
    return dValue;
    }

// -----------------------------------------------------------------

DWORD WINAPI IsapiHeaderHttp (PBYTE  pbData,
                              DWORD  dData,
                              PDWORD pdOffset)
    {
    return IsapiHeaderTest (pbData, dData, pdOffset,
                            abTagHttp, TRUE);
    }

// -----------------------------------------------------------------

DWORD WINAPI IsapiHeaderFind (PBYTE  pbData,
                              DWORD  dData,
                              PDWORD pdOffset,
                              PBYTE  pbTag)
    {
    DWORD dOffset = 0;
    DWORD dValue  = 0;

    if ((pbTag != NULL) && pbTag [0] &&
        IsapiHeaderHttp (pbData, dData, &dOffset))
        {
        while ((!IsapiHeaderStop (pbData, dData, dOffset))
               &&
               (!(dValue = IsapiHeaderTest (pbData, dData, &dOffset,
                                            pbTag, FALSE))));
        }
    if (pdOffset != NULL) *pdOffset = dOffset;
    return dValue;
    }

// -----------------------------------------------------------------

DWORD WINAPI IsapiHeaderValue (PBYTE        pbData,
                               DWORD        dData,
                               PDWORD       pdOffset,
                               PBYTE        pbTag,
                               PISAPI_VALUE piv,
                               BOOL         fCase)
    {
    PBYTE pbValue;
    DWORD dValue, i, j;
    DWORD dOffset = 0;
    DWORD dIndex  = MAXDWORD;

    if ((piv != NULL) &&
        (dValue = IsapiHeaderFind (pbData, dData, &dOffset, pbTag)))
        {
        dValue = IsapiHeaderToken (pbData, dData, dValue);

        for (i = 0; (pbValue = piv [i].pbValue) != NULL; i++)
            {
            for (j = 0; pbValue [j]; j++)
                {
                if (IsapiHeaderStop (pbData, dData, dValue+j)
                    ||
                    (!IsapiCompare (pbData [dValue+j], pbValue [j],
                                    fCase)))
                    {
                    break;
                    }
                }
            if (!pbValue [j]) break;
            }
        if (pbValue != NULL)
            {
            j = IsapiHeaderToken (pbData, dData, dValue+j);
            if (IsapiHeaderStop (pbData, dData, j)) dIndex = i;
            }
        }
    if (pdOffset != NULL) *pdOffset = dOffset;
    return dIndex;
    }

// =================================================================
// ISAPI SIMULATOR
// =================================================================

PISAPI_TEST WINAPI IsapiTestLoad (PBYTE pbPath)
    {
    PISAPI_TEST pit = NULL;

    if ((pit = IsapiMemoryCreate (ISAPI_TEST_)) != NULL)
        {
        if (((pit->hModule =
              (pbPath != NULL ? LoadLibraryA (pbPath) : NULL))
             != NULL)
            &&
            ((pit->GetExtensionVersion =
              GetProcAddress (pit->hModule, "GetExtensionVersion"))
             != NULL)
            &&
            ((pit->HttpExtensionProc =
              GetProcAddress (pit->hModule, "HttpExtensionProc"))
             != NULL))
            {
            pit->TerminateExtension =
                GetProcAddress (pit->hModule, "TerminateExtension");

            pit->pData = NULL;
            pit->dData = 0;
            pit->dSize = 0;
            }
        else
            {
            pit = IsapiTestUnload (pit);
            }
        }
    return pit;
    }

// -----------------------------------------------------------------

PISAPI_TEST WINAPI IsapiTestUnload (PISAPI_TEST pit)
    {
    BOOL fOk = FALSE;

    if (pit != NULL)
        {
        if (pit->hModule != NULL) FreeLibrary (pit->hModule);
        IsapiMemoryDestroy (pit);
        }
    return NULL;
    }

// -----------------------------------------------------------------

BOOL WINAPI IsapiTestStart (PISAPI_TEST       pit,
                            PHSE_VERSION_INFO phvi)
    {
    HSE_VERSION_INFO hvi;
    BOOL             fOk = FALSE;

    if ((pit != NULL) && (pit->GetExtensionVersion != NULL))
        {
        fOk = pit->GetExtensionVersion (phvi != NULL ? phvi : &hvi);
        }
    return fOk;
    }

// -----------------------------------------------------------------

BOOL WINAPI IsapiTestStop (PISAPI_TEST pit)
    {
    BOOL fOk = FALSE;

    if (pit != NULL)
        {
        fOk = (pit->TerminateExtension == NULL) ||
              (pit->TerminateExtension (HSE_TERM_MUST_UNLOAD));
        }
    return fOk;
    }

// -----------------------------------------------------------------

BOOL WINAPI IsapiTestRun (PISAPI_TEST pit,
                          PBYTE       pbPath,
                          PBYTE       pbInfo,
                          PBYTE       pbTranslated,
                          PBYTE       pbQuery,
                          BOOL        fPost)
    {
    EXTENSION_CONTROL_BLOCK ecb;
    PVOID                   pData;
    DWORD                   dBytes, dVersion;
    BOOL                    fOk = FALSE;

    if ((pit     != NULL) && (pbPath                 != NULL) &&
        (pbInfo  != NULL) && (pbTranslated           != NULL) &&
        (pbQuery != NULL) && (pit->HttpExtensionProc != NULL) &&
        ((pData = IsapiMemoryCreate (ISAPI_DATA))    != NULL))
        {
        dBytes     = lstrlenA (pbQuery);
        dVersion   = MAKELONG (ISAPI_VERSION_MINOR,
                               ISAPI_VERSION_MAJOR);
        pit->pData = pData;
        pit->dData = 0;
        pit->dSize = ISAPI_DATA;

        ecb.cbSize                = sizeof (ecb);
        ecb.dwVersion             = dVersion;
        ecb.ConnID                = pit;
        ecb.dwHttpStatusCode      = HTTP_STATUS_OK;
        ecb.lpszLogData [0]       = 0;
        ecb.lpszPathInfo          = pbInfo;
        ecb.lpszPathTranslated    = pbTranslated;
        ecb.GetServerVariable     = IsapiTestGetServerVariable;
        ecb.WriteClient           = IsapiTestWriteClient;
        ecb.ReadClient            = IsapiTestReadClient;
        ecb.ServerSupportFunction = IsapiTestServerSupportFunction;

        if (fPost)
            {
            ecb.lpszMethod        = abMethodPost;
            ecb.lpszQueryString   = abNull;
            ecb.cbTotalBytes      = dBytes;
            ecb.cbAvailable       = dBytes;
            ecb.lpbData           = pbQuery;
            ecb.lpszContentType   = abContentType;
            }
        else
            {
            ecb.lpszMethod        = abMethodGet;
            ecb.lpszQueryString   = pbQuery;
            ecb.cbTotalBytes      = 0;
            ecb.cbAvailable       = 0;
            ecb.lpbData           = abNull;
            ecb.lpszContentType   = abNull;
            }
        if (pit->HttpExtensionProc (&ecb) == HSE_STATUS_SUCCESS)
            {
            fOk = (pit->pData != NULL);
            }
        else
            {
            pit->pData = IsapiMemoryDestroy (pit->pData);
            pit->dData = 0;
            pit->dSize = 0;
            }
        }
    return fOk;
    }

// -----------------------------------------------------------------

BOOL WINAPI IsapiTestDisplay (PISAPI_TEST pit)
    {
    BYTE  abPath [MAX_PATH];
    PBYTE pbData, pbFile;
    DWORD dData, dOffset, dIndex;
    BOOL  fOk = FALSE;

    if ((pit != NULL) && (pit->pData != NULL))
        {
        pbData  = pit->pData;
        dData   = pit->dData;
        dOffset = 0;

        if ((dIndex = IsapiHeaderValue (pbData, dData, &dOffset,
                                        abTagContentType,
                                        aContentTypes, FALSE))
            != MAXDWORD)
            {
            pbFile  = aContentTypes [dIndex].pbData;
            dOffset = IsapiHeaderSkip (pbData, dData, dOffset);

            pbData += dOffset;
            dData  -= dOffset;
            }
        else
            {
            pbFile = abFileDefault;
            }
        if (IsapiFilePath (pit->hModule, pbFile, abPath, MAX_PATH)&&
            IsapiFileSave (abPath, pbData, dData))
            {
            fOk = (ShellExecuteA (NULL, abShellOpen, abPath,
                                  NULL, NULL, SW_SHOWNORMAL)
                   > (HINSTANCE) HINSTANCE_ERROR);
            }
        }
    return fOk;
    }

// -----------------------------------------------------------------

BOOL WINAPI IsapiTestOut (PEXTENSION_CONTROL_BLOCK pecb,
                          PVOID                    pData,
                          DWORD                    dData)
    {
    DWORD n1, n2;
    BOOL  fOk = FALSE;

    if ((pecb != NULL) && (pData != NULL))
        {
        n1 = n2 = (dData ? dData : lstrlenA (pData));

        fOk = (!n2) || (pecb->WriteClient (pecb->ConnID, pData, &n2,
                                           HSE_IO_SYNC)
                        && (n1 == n2));
        }
    return fOk;
    }

// -----------------------------------------------------------------

BOOL WINAPI IsapiTestGetServerVariable (HCONN  hConn,
                                        PSTR   psVariableName,
                                        PVOID  pBuffer,
                                        PDWORD pdSize)
    {
    DWORD dStatus = ERROR_INVALID_PARAMETER;

    if (pdSize != NULL)
        {
        *pdSize = 0;
        dStatus = ERROR_NO_DATA;
        }
    SetLastError (dStatus);
    return (dStatus == ERROR_SUCCESS);
    }

// -----------------------------------------------------------------

BOOL WINAPI IsapiTestWriteClient (HCONN  hConn,
                                  PVOID  pBuffer,
                                  PDWORD pdSize,
                                  DWORD  dReserved)
    {
    DWORD       dData, dSize;
    PISAPI_TEST pit     = hConn;
    DWORD       dStatus = ERROR_INVALID_PARAMETER;

    if (pdSize != NULL)
        {
        if ((pBuffer != NULL) &&
            (pit     != NULL) && (pit->pData != NULL))
            {
            dData = pit->dData + *pdSize;
            dSize = pit->dSize;

            if (dData >= pit->dData)
                {
                if (dData > pit->dSize)
                    {
                    dSize = (((dData-1)/ISAPI_DATA)+1)*ISAPI_DATA;
                    }
                if (dData <= dSize)
                    {
                    if (dSize > pit->dSize)
                        {
                        pit->pData = IsapiMemoryResize (pit->pData,
                                                        dSize);
                        }
                    if (pit->pData != NULL)
                        {
                        IsapiMemoryCopy ((PBYTE) pit->pData +
                                                 pit->dData,
                                         pBuffer, *pdSize);

                        pit->dData = dData;
                        pit->dSize = dSize;

                        dStatus = ERROR_SUCCESS;
                        }
                    else
                        {
                        pit->dData = 0;
                        pit->dSize = 0;

                        dStatus = ERROR_NOT_ENOUGH_MEMORY;
                        }
                    }
                }
            }
        if (dStatus != ERROR_SUCCESS) *pdSize = 0;
        }
    SetLastError (dStatus);
    return (dStatus == ERROR_SUCCESS);
    }

// -----------------------------------------------------------------

BOOL WINAPI IsapiTestReadClient (HCONN  hConn,
                                 PVOID  pBuffer,
                                 PDWORD pdSize)
    {
    DWORD dStatus = ERROR_INVALID_PARAMETER;

    if (pdSize != NULL)
        {
        *pdSize = 0;
        dStatus = ERROR_SUCCESS;
        }
    SetLastError (dStatus);
    return (dStatus == ERROR_SUCCESS);
    }

// -----------------------------------------------------------------

BOOL WINAPI IsapiTestServerSupportFunction (HCONN  hConn,
                                            DWORD  dRequest,
                                            PVOID  pBuffer,
                                            PDWORD pdSize,
                                            PDWORD pdType)
    {
    DWORD dStatus = ERROR_INVALID_PARAMETER;

    if (pdSize != NULL)
        {
        *pdSize = 0;
        dStatus = ERROR_NO_DATA;
        }
    SetLastError (dStatus);
    return (dStatus == ERROR_SUCCESS);
    }

// -----------------------------------------------------------------

PISAPI_QUERY WINAPI IsapiTestQuery (PBYTE pbData)
    {
    DWORD        i, j, k, l;
    BOOL         fQuote;
    BYTE         abFolder [MAX_PATH];
    BYTE         abInfo   [MAX_PATH];
    PBYTE        pbFilePart;
    PISAPI_QUERY piq = NULL;

    if ((pbData != NULL) &&
        ((piq = IsapiMemoryCreate (ISAPI_QUERY_)) != NULL))
        {
        piq->fPost = FALSE;

        for (i = 0; pbData [i] == ' '; i++);

        if (fQuote = (pbData [i] == '"'))
            {
            for (i++; pbData [i] == ' '; i++);
            }
        if (pbData [i] == '-')
            {
            for (j = ++i; pbData [j] && (pbData [j] != ' '); j++);

            piq->fPost = IsapiMatch (pbData+i, j-i,
                                     "post", MAXDWORD, FALSE);

            for (i = j; pbData [i] == ' '; i++);
            }
        for (j = i; pbData [j] && (pbData [j] != '/')
                               && (pbData [j] != '?'); j++);

        for (k = j; (k > i) && (pbData [k-1] == ' '); k--);

        for (l = k; (l > i) && (pbData [l-1] != '\\')
                            && (pbData [l-1] != ':' ); l--);
                                
        IsapiCopy (pbData+l, k-l, piq->abFile, MAX_PATH);

        if ((l-i >= 2) && (pbData [l-1] == '\\')
                       && (pbData [l-2] != ':' )) l--;

        IsapiCopy (pbData+i, l-i, abFolder, MAX_PATH);

        i = j;

        if (pbData [i] == '/')
            {
            while (pbData [j] && (pbData [j] != '?')) j++;
            }
        for (k = j; (k > i) && (pbData [k-1] == ' '); k--);

        IsapiCopy (pbData+i, k-i, piq->abInfo, MAX_PATH);
        IsapiCopy (pbData+i, k-i,      abInfo, MAX_PATH);

        for (i = 0; abInfo [i]; i++)
            {
            if (abInfo [i] == '/') abInfo [i] = '\\';
            }
        i = (pbData [j] == '?' ? j+1 : j);

        for (j = i; pbData [j] &&
                    ((!fQuote) || (pbData [j] != '"')); j++);

        IsapiCopy (pbData+i, j-i, piq->abQuery, MAX_QUERY);

        if ((i = GetCurrentDirectoryA (MAX_PATH, piq->abHome)) &&
            (i < MAX_PATH))
            {
            if (abFolder [0])
                {
                j = GetFullPathNameA (abFolder,
                                      MAX_PATH, piq->abFolder,
                                      &pbFilePart);
                }
            else
                {
                lstrcpynA (piq->abFolder, piq->abHome, MAX_PATH);
                j = i;
                }
            if (abInfo [0])
                {
                k = GetFullPathNameA (abInfo,
                                      MAX_PATH, piq->abTranslated,
                                      &pbFilePart);
                }
            else
                {
                piq->abTranslated [0] = 0;
                k = i;
                }
            }
        if (!(i && (i < MAX_PATH) &&
              j && (j < MAX_PATH) &&
              k && (k < MAX_PATH)))
            {
            piq = IsapiMemoryDestroy (piq);
            }
        }
    return piq;
    }

// -----------------------------------------------------------------

DWORD WINAPI IsapiTest (PISAPI_QUERY piq,
                        PVOID       *ppData,
                        PDWORD       pdStatus)
    {
    PISAPI_TEST pit;
    DWORD       dStatus = ISAPI_STATUS_UNDEFINED;
    DWORD       dData   = 0;

    if (ppData != NULL) *ppData = NULL;

    dStatus = ISAPI_STATUS_ERROR_PARAMETER;
    if (piq != NULL)
        {
        dStatus = ISAPI_STATUS_ERROR_LOAD;
        if (SetCurrentDirectoryA (piq->abFolder) &&
            ((pit = IsapiTestLoad (piq->abFile)) != NULL))
            {
            dStatus = ISAPI_STATUS_ERROR_START;
            if (IsapiTestStart (pit, NULL))
                {
                dStatus = ISAPI_STATUS_ERROR_RUN;
                if (IsapiTestRun (pit, piq->abFile,
                                  piq->abInfo, piq->abTranslated,
                                  piq->abQuery, piq->fPost))
                    {
                    dStatus = ISAPI_STATUS_OK;
                    if (ppData != NULL)
                        {
                        *ppData = pit->pData;
                        dData   = pit->dData;
                        }
                    else
                        {
                        if ((!pit->dData) || IsapiTestDisplay (pit))
                            {
                            dData = pit->dData;
                            }
                        else
                            {
                            dStatus = ISAPI_STATUS_ERROR_DISPLAY;
                            }
                        IsapiMemoryDestroy (pit->pData);
                        }
                    }
                }
            IsapiTestStop   (pit);
            IsapiTestUnload (pit);
            }
        SetCurrentDirectoryA (piq->abHome);
        }
    if (pdStatus != NULL) *pdStatus = dStatus;
    return dData;
    }

// =================================================================
// ISAPI TEST INTERFACE
// =================================================================

BOOL WINAPI GetExtensionVersion (PHSE_VERSION_INFO phvi)
    {
    BOOL fOk = FALSE;

    if (phvi != NULL)
        {
        phvi->dwExtensionVersion = MAKELONG (HSE_VERSION_MINOR,
                                             HSE_VERSION_MAJOR);

        lstrcpynA (phvi->lpszExtensionDesc, abName,
                   HSE_MAX_EXT_DLL_NAME_LEN);

        fOk = TRUE;
        }
    return fOk;
    }

// -----------------------------------------------------------------

DWORD WINAPI HttpExtensionProc (PEXTENSION_CONTROL_BLOCK pecb)
    {
    DWORD dStatus = HSE_STATUS_ERROR;

    if (IsapiTestOut (pecb, abHtmlStart,       0) &&
        IsapiTestOut (pecb, SA(ABOUT_CAPTION), 0) &&
        IsapiTestOut (pecb, abNewLine,         0) &&
        IsapiTestOut (pecb, abNewLine,         0) &&
        IsapiTestOut (pecb, SA(ABOUT_TEXT1),   0) &&
        IsapiTestOut (pecb, abNewLine,         0) &&
        IsapiTestOut (pecb, SA(ABOUT_TEXT2),   0) &&
        IsapiTestOut (pecb, abNewLine,         0) &&
        IsapiTestOut (pecb, SA(ABOUT_TEXT3),   0) &&
        IsapiTestOut (pecb, abNewLine,         0) &&
        IsapiTestOut (pecb, SA(ABOUT_TEXT4),   0) &&
        IsapiTestOut (pecb, abHtmlStop,        0))
        {
        dStatus = HSE_STATUS_SUCCESS;
        }
    return dStatus;
    }

// -----------------------------------------------------------------

BOOL WINAPI TerminateExtension (DWORD dFlags)
    {
    return TRUE;
    }

// =================================================================
// DLL MANAGEMENT
// =================================================================

BOOL WINAPI DllMain (HINSTANCE hInstance,
                     DWORD     dReason,
                     PVOID     pReserved)
    {
    BOOL fOk = TRUE;

    if (dReason == DLL_PROCESS_ATTACH)
        {
        ghInstance = hInstance;
        }
    return fOk;
    }

// =================================================================
// END OF PROGRAM
// =================================================================
