
// __________________________________________________________
//
//                          isolde.c
//   Internet Server OffLine Debugger for Extensions V1.01
//                05-02-1999 Sven B. Schreiber
//                       sbs@orgon.com
// __________________________________________________________

#include "isolde.h"

// =================================================================
// DISCLAIMER
// =================================================================

/*

This software is provided "as is" and any express or implied
warranties, including, but not limited to, the implied warranties of
merchantibility and fitness for a particular purpose are disclaimed.
In no event shall the author Sven B. Schreiber be liable for any
direct, indirect, incidental, special, exemplary, or consequential
damages (including, but not limited to, procurement of substitute
goods or services; loss of use, data, or profits; or business
interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software,
even if advised of the possibility of such damage.

*/

// =================================================================
// REVISION HISTORY
// =================================================================

/*

03-17-1999 V1.00 Original version (SBS).

05-02-1999 V1.01 Update (SBS).

    Modified WinMain() to use the new command line parser introduced
    by IsapiSim.dll V1.01.

*/

// =================================================================
// GLOBAL STRINGS
// =================================================================

BYTE abCaption [] = SA(MAIN_CAPTION);
BYTE abNoData  [] = "This request didn't return any data.";
BYTE abUsage   [] = "Usage:\n" SA(MAIN_MODULE)
                    " [-post] <DLL path>[/<path info>][?<query>]";

// -----------------------------------------------------------------

BYTE abError          [] = "ISAPI simulator error #%lu:\n%s";
BYTE abErrorParameter [] = "Invalid API parameter.";
BYTE abErrorLoad      [] = "Failed to load the ISAPI DLL.";
BYTE abErrorStart     [] = "GetExtensionVersion() failed.";
BYTE abErrorRun       [] = "HttpExtensionProc() failed.";
BYTE abErrorDisplay   [] = "Unable to display the returned data.";
BYTE abErrorUnknown   [] = "Unknown error condition.";

// =================================================================
// MAIN PROGRAM
// =================================================================

int WINAPI WinMain (HINSTANCE hInstance,
                    PCMDLINE  pcl,
                    PTBYTE    ptCmdLine,
                    int       iShowCmd)

    {
    PISAPI_QUERY piq;
    PBYTE        pbError;
    DWORD        dStatus = ISAPI_STATUS_UNDEFINED;

    if (pcl->argc < 2)
        {
        IsapiBox (NULL, MB_OK | MB_ICONINFORMATION,
                  abCaption, abUsage);
        }
    else
        {
        piq = IsapiTestQuery (ptCmdLine);

        if (!IsapiTest (piq, NULL, &dStatus))
            {
            if (dStatus == ISAPI_STATUS_OK)
                {
                IsapiBox (NULL, MB_OK | MB_ICONINFORMATION,
                          abCaption, abNoData);
                }
            else
                {
                switch (dStatus)
                    {
                    case ISAPI_STATUS_ERROR_PARAMETER:
                        pbError = abErrorParameter;
                        break;

                    case ISAPI_STATUS_ERROR_LOAD:
                        pbError = abErrorLoad;
                        break;

                    case ISAPI_STATUS_ERROR_START:
                        pbError = abErrorStart;
                        break;

                    case ISAPI_STATUS_ERROR_RUN:
                        pbError = abErrorRun;
                        break;

                    case ISAPI_STATUS_ERROR_DISPLAY:
                        pbError = abErrorDisplay;
                        break;

                    default:
                        pbError = abErrorUnknown;
                        break;
                    }
                IsapiBox (NULL, MB_OK | MB_ICONINFORMATION,
                          abCaption, abError, dStatus, pbError);
                }
            }
        IsapiMemoryDestroy (piq);
        }
    return dStatus;
    }

// =================================================================
// END OF PROGRAM
// =================================================================
