NR == 1 {
    # --- Initial default values ---
    BaseStr = "SAMPLE"
    TitleStr = "Sample Help"
    SectionStr = TitleStr
    # Assume output file has same base name as current input file
    FileStr = FILENAME
    Pos = index(FileStr, ".")
    if (Pos) FileStr = substr(FileStr, 1, Pos - 1)
    ContentStr = FileStr ".CNT"
    ProjectStr = FileStr ".HPJ"
    # --- Write beginning of RTF file ---
    print "{"
    print "\\rtf1\\ansi\\deff0\\deflang1024\\plain\\sb60\\sa60"
    print "{\n\\fonttbl"
    print "{\\f0\\froman Times New Roman;}"
    print "{\\f1\\fswiss Tahoma;}"
    print "{\\f2\\fmodern Courier New;}\n}"
    print "{\n\\colortbl;"
    print "\\red0\\green0\\blue0;" # Black
    print "\\red0\\green128\\blue128;" # Cyan
    print "\\red128\\green0\\blue0;" # Red
    print "\\red255\\green255\\blue255;\n}" # White
    print ":base " FileStr ".HLP" > ContentStr
} # First line

# --- Record title ---

/^\.Title / {
    match($0, / [^ ]/)
    TitleStr = substr($0, RSTART + 1)
} # Title

# --- Record copyright ---

/^\.Copyright / {
    match($0, / [^ ]/)
    CopyrightStr = "Contents copyright \251 " \
        substr($0, RSTART + 1)
} # Copyright

# --- Pascal topic ID unit ---

/^\.Pascal +[^ ]+/ {
    match($0, / [^ ]/)
    UnitStr = substr($0, RSTART + 1)
    PascalStr = UnitStr ".PAS"
    print "unit " UnitStr ";" > PascalStr
    print "" >> PascalStr
    print "{Topic IDs for " FileStr ".HLP}" > PascalStr
    print "" >> PascalStr
    print "interface" >> PascalStr
    print "" >> PascalStr
    print "const" >> PascalStr
    print "" >> PascalStr
} # Pascal

# --- C topic ID header file ---

/^\.C +[^ ]+/ {
    match($0, / [^ ]/)
    CStr = substr($0, RSTART + 1) ".H"
    print "/* Topic IDs for " FileStr ".HLP */" > CStr
    print "" >> CStr
} # C

# --- Display compile date in version help ---

/^\.Date$/ {
    DateStr = " %date"
} # Date

# --- Print end-of-paragraph marker if needed ---

function ParagraphEnd() {
    if (ParOut) {
        if (ColCount) print "\\cell\\row\\trowd"
        else print "\\par"
        print "}"
        ColCount = 0
        ParOut = 0
        LineOut = 0
        ExampleMode = 0
    } # if
} # ParagraphEnd

# --- Print end-of-page marker if needed ---

function PageEnd() {
    delete KeyList
    ParagraphEnd()
    if (PageOut) {
        print "\\page"
        PageOut = 0
    } # if
} # PageEnd

# --- Print link to popup or topic ---

function Link(AStr, BStr, CStr) {
    Pos = index(RefStr, "=")
    if (Pos) {
        TopicStr = substr(RefStr, 1, Pos - 1)
        JumpStr = substr(RefStr, Pos + 1)
    } # if
    else {
        TopicStr = RefStr
        JumpStr = RefStr
    } # else
    gsub(/_/, " ", TopicStr)
    NewStr = substr($0, 1, RSTART - 1) \
        AStr TopicStr BStr JumpStr CStr \
        substr($0, RSTART + RLENGTH)
    $0 = NewStr
} # Link

# --- Translate control sequences ---

function Condition() {
    gsub(/\\/, "\\\\")
    gsub(/'/, "\\rquote ")
    gsub(/{<}/, "\\lquote ")
    gsub(/{>}/, "\\rquote ")
    gsub(/{<<}/, "\\ldblquote ")
    gsub(/{>>}/, "\\rdblquote ")
    gsub(/{B}/, "\\b ")
    gsub(/{b}/, "\\b0 ")
    gsub(/{U}/, "\\ul ")
    gsub(/{u}/, "\\ul0 ")
    gsub(/{I}/, "\\i ")
    gsub(/{i}/, "\\i0 ")
    while (match($0, /{K}[A-Za-z0-9_]+/)) {
        KeyStr = substr($0, RSTART, RLENGTH)
        gsub(/{K}/, "", KeyStr)
        gsub(/_/, " ", KeyStr)
        NewStr = substr($0, 1, RSTART - 1) KeyStr \
            substr($0, RSTART + RLENGTH)
        $0 = NewStr
        if (! KeyList[KeyStr]) print " {K{\\footnote " KeyStr "}}"
        KeyList[KeyStr] = 1
    } # while
    gsub(/{/, "\\{")
    gsub(/}/, "\\}")
    while (match($0, /\\{T\\}[A-Za-z0-9_=]+/)) {
        RefStr = substr($0, RSTART, RLENGTH)
        gsub(/\\{T\\}/, "", RefStr)
        Link("{\\strike ", "}{\\v IDH_", "}")
    } # while
    while (match($0, /\\{P\\}[A-Za-z0-9_=]+/)) {
        RefStr = substr($0, RSTART, RLENGTH)
        gsub(/\\{P\\}/, "", RefStr)
        Link("{{\\cf2\\ul ", "}{\\v *Pop_", "}}")
    } # while
} # Condition

# --- Write options to project file ---

function ProjectOptions() {
    print ":title " TitleStr >> ContentStr
    print "[OPTIONS]" > ProjectStr
    print "LCID=0x409 0x0 0x0" >> ProjectStr
    print "TITLE=" TitleStr >> ProjectStr
    print "ERRORLOG=" FileStr ".LOG" >> ProjectStr
    print "HLP=" FileStr ".HLP" >> ProjectStr
    if (CopyrightStr != "") \
        print "COPYRIGHT=" CopyrightStr DateStr >> ProjectStr
    print "" >> ProjectStr
    print "[FILES]" >> ProjectStr
    # Output must be directed to FileStr ".RTF"
    print FileStr ".RTF" >> ProjectStr
    print "" >> ProjectStr
    print "[MAP]" >> ProjectStr
    OptionInit = 1
} # ProjectOptions

# --- Record section text ---

/^\.Section +[^ ]+/ {
    if (! OptionInit) ProjectOptions()
    Condition()
    match($0, / [^ ]/)
    SectionStr = substr($0, RSTART + 1)
    TmpStr = SectionStr
    gsub(/\\rquote /, "'", TmpStr)
    print "1 " TmpStr >> ContentStr
} # Section

# --- Prepare link and title for topic or popup ---

# $0 is something like ".Topic This is a topic {A}"
# PrefixStr is something like "IDH"
# IdStr becomes "IDH_This_is_a_topic_A"
# TopicStr becomes "This is a topic"

function TitleLink(PrefixStr) {
    match($0, / [^ ]/)
    TopicStr = substr($0, RSTART + 1)
    IdStr = PrefixStr "_" TopicStr
    gsub(/\\[^ ]/, "", IdStr)
    # gsub(/{/, "", IdStr)
    # gsub(/}/, "", IdStr)
    # gsub(/"/, "", IdStr)
    # gsub(/'/, "", IdStr)
    gsub(/  */, "_", IdStr)
    gsub(/[^A-Za-z0-9_]/, "", IdStr)
    while (match(TopicStr, /{[^}]*}/)) {
        TopicStr = substr(TopicStr, 1, RSTART - 1) \
            substr(TopicStr, RSTART + RLENGTH + 1)
    } # while
    gsub(/   */, " ", TopicStr)
} # TitleLink

# --- Process a topic line ---

/(^\.Topic)( |Hidden )( *[^ ]+)/ {
    Hidden = ($0 ~ /^\.TopicHidden /)
    if (! OptionInit) ProjectOptions()
    PageEnd()
    TopicId++
    TitleLink("IDH")
    print ""
    print "${\\footnote " TopicStr "}"
    print "#{\\footnote " IdStr "}"
    if (! Hidden) {
        printf("+{\\footnote %03d}\n", TopicId)
        print "K{\\footnote " TopicStr "}"
    } # if
    print "{"
    print "\\f1\\fs40\\qc\\cf2\\keepn\\sb40\\sa40"
    print TopicStr
    print "\\par"
    print "}"
    print "{"
    print "\\fs28\\f1\\b\\qc\\cf1"
    print "\\keepn\\sb0\\sa40\\scaps"
    print SectionStr
    print "\\par"
    print "}"
    print IdStr " " TopicId >> ProjectStr
    if (PascalStr != "") \
        print "  " IdStr " = " TopicId ";" >> PascalStr
    if (CStr != "") print "#define " IdStr " " TopicId >> CStr
    if (! Hidden) print "2 " TopicStr "=" IdStr >> ContentStr
} # Topic

# --- Process a popup topic line ---

/^\.Popup +[^ ]+/ {
    PageEnd()
    print ""
    # Don't use IDH_ namespace
    TitleLink("Pop")
    print "#{\\footnote " IdStr "}"
    PageOut = 1
} # Popup

# --- Include 16 color bitmap ---

/(^\.Bitmap)( |Indent )( *[^ ]+)/ {
    ParagraphEnd()
    if ($0 ~ /.BitmapIndent /) IndentStr = "\\tx360\\tab"
    else IndentStr = ""
    match($0, / [^ ]/)
    print "{" IndentStr "\\{bmct " \
        substr($0, RSTART + 1) ".bmp\\}}"
    print "\\par"
    PageOut = 1
} # Bitmap

# --- Process first line of paragraph text ---

/^\.Text +[^ ]+/ {
    ParagraphEnd()
    Condition()
    match($0, / [^ ]/)
    print "{"
    print substr($0, RSTART + 1)
    ParOut = 1
    PageOut = 1
} # Text

# --- Process first line of example text ---

/^\.Example +[^ ]+/ {
    ParagraphEnd()
    Condition()
    match($0, / [^ ]/)
    print "{"
    # print "\\f2\\fs22\\keep\\li360"
    print "\\f2\\fs22\\keep\\li360\\sb0\\sa0"
    print substr($0, RSTART + 1)
    ExampleMode = 1
    LineOut = 1
    ParOut = 1
    PageOut = 1
} # Example

# --- Process first line of bulleted text ---

/^\.Bullet +[^ ]+/ {
    ParagraphEnd()
    Condition()
    print "{"
    print "\\li360\\fi-240\\tx360\\bullet\\tab"
    match($0, / [^ ]/)
    print substr($0, RSTART + 1)
    ParOut = 1
    PageOut = 1
} # Bullet

# --- Process beginning of columnar table ---

/^\.Table *[0-9 ]+$/ { # Later, support .TableIndent
    ParagraphEnd()
    # Parameters are column widths in percentage of window width
    ColCount = NF - 1
    if (ColCount) {
        Pos = 0
        print "{"
        printf "\\trqc"
        for (J = 2; J <= NF; J++) {
            Wd = $J + 0
            Pos += (Wd <= 0) ? 100 : Wd
            printf "\\cellx%d", int(Pos * 86)
        } # J
        print ""
    } # if
    Col = 0 # Zero only when no cell ouput yet
    ParOut = 1
    PageOut = 1
} # Table

# --- Process table cell text ---

/^\.Cell/ {
    Condition()
    if (Col) {
        if (Col == 1) print "\\cell\\row\\intbl"
        else print "\\cell"
    } # if
    else {
        print "\\intbl"
        Col = 1
    } # else
    Col++
    if (Col > ColCount) Col = 1
    if (match($0, / [^ ]/)) print substr($0, RSTART + 1)
} # Cell

# --- Process first line of indented text ---

/^\.Indent +[^ ]+/ {
    ParagraphEnd()
    Condition()
    print "{"
    print "\\li360"
    match($0, / [^ ]/)
    print substr($0, RSTART + 1)
    ParOut = 1
    PageOut = 1
} # Indent

# --- Process subsequent lines of text ---

! (/^\./ || /^ *$/) {
    if (LineOut && ExampleMode) print "\\line"
    Condition()
    if (! ExampleMode) printf " "
    sub(/^ */, "")
    print
    LineOut = 1
} # Paragraph text

# --- Wrap up ---

END {
    PageEnd()
    print "}" # End of RTF file
    print "" >> ProjectStr
    print "[WINDOWS]" >> ProjectStr
    print "Main=\"\",(66,71,892,894)," \
        "60672,,(r12648447),f2" >> ProjectStr
    print "" >> ProjectStr
    print "[CONFIG]" >> ProjectStr
    print "BrowseButtons()" >> ProjectStr
    print "SetPopupColor(255,255,192)" >> ProjectStr
    if (PascalStr != "") {
        print "" >> PascalStr
        print "implementation" >> PascalStr
        print "" >> PascalStr
        print "end." >> PascalStr
    } # if
} # "END"
