// mymsg.c

#define _WIN32_WINNT 0x0500
#include "windows.h"

#define TMR 0
#define MSG 1

typedef struct _HDR_DATA {
    HANDLE hEvent;
    DWORD  dwData;
} HDR_DATA, *PHDR_DATA;

typedef struct _TMR_DATA {
  HDR_DATA Hdr;
} TMR_DATA, *PTMR_DATA;

typedef struct _MSG_DATA {
    HDR_DATA Hdr;
    HWND     hWnd;
    UINT     Msg;
    WPARAM   wParam;
    LPARAM   lParam;
} MSG_DATA, *PMSG_DATA;

VOID CALLBACK TimerCallback(PVOID, BOOLEAN);
DWORD CALLBACK MsgCallback(PVOID);
VOID CALLBACK CleanupCallback(PVOID, BOOLEAN);

HANDLE hTimerQueue = NULL;

//----------------------------------------------------------------- 
BOOL WINAPI DllEntryPoint(HINSTANCE hInstance, DWORD dwReason,
                          LPVOID pContext)
{
    switch (dwReason) {
        case DLL_PROCESS_ATTACH:
            hTimerQueue = CreateTimerQueue();
            break;

        case DLL_PROCESS_DETACH:
            if (hTimerQueue) DeleteTimerQueue(hTimerQueue);
            break;

        case DLL_THREAD_ATTACH:
        case DLL_THREAD_DETACH:
            break;
    }

    return TRUE;

} // DllEntryPoint

//----------------------------------------------------------------- 
LRESULT WINAPI MySendMessageTimeout(HWND hWnd, UINT Msg, 
                                    WPARAM wParam, LPARAM lParam, 
                                    DWORD dwTimeout, DWORD *pResult)
{
    DWORD status;
    HANDLE Events[2];
    HANDLE hTimer = NULL;
    PTMR_DATA pTimerData = NULL;
    PMSG_DATA pMsgData = NULL;
    HANDLE hCleanupEvent = NULL;

    // Allocate context data and queue a work item to send msg

    pMsgData = malloc(sizeof(MSG_DATA));
    if (pMsgData == NULL) {
        return ERROR_NOT_ENOUGH_MEMORY;
    }

    Events[MSG] = CreateEvent(NULL, TRUE, FALSE, NULL);
    pMsgData->Hdr.hEvent        = Events[MSG];
    pMsgData->Hdr.dwData        = 0;
    pMsgData->hWnd              = hWnd;
    pMsgData->Msg               = Msg;
    pMsgData->lParam            = lParam;
    pMsgData->wParam            = wParam;

    QueueUserWorkItem(MsgCallback, pMsgData, WT_EXECUTEDEFAULT);

    // allocate context data and queue a timer

    pTimerData = malloc(sizeof(TMR_DATA));
    if (pTimerData == NULL) {
        return ERROR_NOT_ENOUGH_MEMORY;
    }

    Events[TMR] = CreateEvent(NULL, TRUE, FALSE, NULL);
    pTimerData->Hdr.hEvent        = Events[TMR];
    pTimerData->Hdr.dwData        = 0;

    if (!CreateTimerQueueTimer(&hTimer, hTimerQueue, TimerCallback,
                               (PVOID)pTimerData, dwTimeout, 0,
                               WT_EXECUTEONLYONCE)) {
        return GetLastError();
    }

    // now wait for either the message or timer to complete

    status = WaitForMultipleObjects(2, Events, FALSE, INFINITE);

    if ((status - WAIT_OBJECT_0) == MSG) {

        // Message finished; free message resources and queue work 
        // item to free timer resources.

        DeleteTimerQueueTimer(hTimerQueue, hTimer, NULL);
        *pResult = pMsgData->Hdr.dwData;
        CloseHandle(pMsgData->Hdr.hEvent);
        free(pMsgData);
        RegisterWaitForSingleObject(&hCleanupEvent, 
                                    pTimerData->Hdr.hEvent, 
                                    CleanupCallback, 
                                    (PVOID)pTimerData,
                                    INFINITE, WT_EXECUTEONLYONCE);
        UnregisterWaitEx(hCleanupEvent, NULL);
        return TRUE;

    } else if ((status - WAIT_OBJECT_0) == TMR) {

        // Timer finished; free timer resources and queue work 
        // item to free message resources.
    
        DeleteTimerQueueTimer(hTimerQueue, hTimer, NULL);
        CloseHandle(pTimerData->Hdr.hEvent);
        free(pTimerData);
        *pResult = 0xcc;
        RegisterWaitForSingleObject(&hCleanupEvent, 
                                    pMsgData->Hdr.hEvent, 
                                    CleanupCallback,
                                    (PVOID)pMsgData,
                                    INFINITE, WT_EXECUTEONLYONCE);
        UnregisterWaitEx(hCleanupEvent, NULL);
        return FALSE;
    }

    return FALSE;

} // MySendMessageTimeout


//----------------------------------------------------------------- 
VOID CALLBACK TimerCallback(PVOID lpParam, BOOLEAN bFired)
{
    PTMR_DATA pData = (PTMR_DATA)lpParam;
    if (pData) {
        SetEvent(pData->Hdr.hEvent);
    }
    return;

} // MyTimerCallback

//----------------------------------------------------------------- 
DWORD CALLBACK MsgCallback(PVOID lpParam)
{
    PMSG_DATA pData = (PMSG_DATA)lpParam;
    DWORD lResult;

    if (pData) {
        lResult = SendMessage(pData->hWnd, pData->Msg, 
                              pData->wParam, pData->lParam);
        pData->Hdr.dwData = lResult;
        SetEvent(pData->Hdr.hEvent);
    }
    return TRUE;

} // CleanupCallback

//----------------------------------------------------------------- 
VOID CALLBACK CleanupCallback(PVOID lpParam, BOOLEAN bFired)
{
    PHDR_DATA pData = (PHDR_DATA)lpParam;
    if (pData) {
        CloseHandle(pData->hEvent);
        free(pData);
    }
    return;

} // CleanupCallback

